@@ -9,17 +9,23 @@
 import re
 import json
 import pymysql
+import threading
 import pymysql.connections
 from pymysql import converters, FIELD_TYPE
 from pymysql.converters import escape_string
-from . import exceptions
+from . import dqlparse, exceptions
+
 
 # 连接集合
 connections = dict()
-# 全局游标
-global_cursor = None
+# 全局连接
+global_conn = None
 # 缓存信息
 cache_data = dict()
+# 线程锁
+thread_lock = threading.Lock()
+# 线程锁超时时间
+locked_timeout = 10
 
 
 # 事务处理
@@ -30,7 +36,7 @@ class transaction:
     def __init__(self, conn=None):
         # 游标
         if conn is None:
-            self.conn = global_cursor.connection
+            self.conn = global_conn
         else:
             self.conn = conn
 
@@ -45,16 +51,19 @@ class transaction:
         # 装饰器
         def wrapper(*args, **kwargs):
             cls.adjust_level(1)
-            
+
             try:
                 result = conn_or_func(*args, **kwargs)
                 cls.adjust_level(-1)
                 if cls.get_level() == 0:
-                    global_cursor.connection.commit()
+                    global_conn.commit()
                 return result
             except Exception as e:
                 cls.adjust_level(-1)
-                global_cursor.connection.rollback()
+                global_conn.rollback()
+                # 释放锁
+                if thread_lock.locked() is True:
+                    thread_lock.release()
                 raise e
 
         return wrapper
@@ -76,6 +85,9 @@ class transaction:
                 # 回滚后，插入ID和影响行数都改为0
                 cache_data['effected_rows'] = 0
                 cache_data['last_insert_id'] = 0
+                # 释放锁
+                if thread_lock.locked() is True:
+                    thread_lock.release()
                 return False
 
     @classmethod
@@ -90,25 +102,35 @@ class transaction:
 # 查询构建器
 class imysql:
 
-    def __init__(self, cursor=None):
+    # 默认连接（静态变量）
+    default_conn = None
+
+    def __init__(self, conn=None):
+
         # 数据
         self.data = dict()
         # 原生SQL
         self.raw_sql = ''
         # 上一个SQL
         self.last_sql = ''
-        # 游标
-        if cursor is not None:
-            self.cursor = cursor
-        else:
-            self.cursor = global_cursor
-
         # 连接
-        self.conn = self.cursor.connection
-        
+        if conn is not None:
+            self.conn = conn
+        else:
+            self.conn = global_conn
+
+        # 加线程锁
+        thread_lock.acquire(timeout=locked_timeout)
+
         # 断线重连
         self.conn.ping(reconnect=True)
 
+        # 游标
+        self.cursor = self.conn.cursor()
+
+        # 释放锁
+        thread_lock.release()
+
     @classmethod
     def connect(cls, options: dict, name='default'):
         ''' 连接 MySql
@@ -116,37 +138,43 @@ class imysql:
         :param https://pymysql.readthedocs.io/en/latest/modules/connections.html
         '''
 
-        if name in connections:
-            raise exceptions.RuntimeError((400, '请忽重复连接【%s】' % name))
+        global global_conn
+        
+        conv = converters.conversions
+        conv[FIELD_TYPE.NEWDECIMAL] = float
+        conv[FIELD_TYPE.DATE] = str
+        conv[FIELD_TYPE.TIMESTAMP] = str
+        conv[FIELD_TYPE.DATETIME] = str
+        conv[FIELD_TYPE.TIME] = str
 
-        global global_cursor
+        # 默认参数
+        default_options = {
+            'charset': 'utf8',
+            'cursorclass': pymysql.cursors.DictCursor,
+            'conv': conv,
+            'use_unicode': True,
+        }
 
-        if 'charset' not in options:
-            options['charset'] = 'utf8'
-
-        if 'cursorclass' not in options:
-            options['cursorclass'] = pymysql.cursors.DictCursor
-
-        if 'conv' not in options:
-            conv = converters.conversions
-            conv[FIELD_TYPE.NEWDECIMAL] = float
-            conv[FIELD_TYPE.DATE] = str
-            conv[FIELD_TYPE.TIMESTAMP] = str
-            conv[FIELD_TYPE.DATETIME] = str
-            conv[FIELD_TYPE.TIME] = str
-            options['conv'] = conv
+        # 合并参数
+        default_options.update(options)
+        options = default_options
 
         # 连接
-        conn = pymysql.connect(**options)
-        # 保存连接
-        connections[name] = conn
-        
+        if name in connections:
+            # raise exceptions.RuntimeError((400, '请忽重复连接【%s】' % name))
+            conn = connections[name]
+        else:
+            conn = pymysql.connect(**options)
+            # 保存连接
+            connections[name] = conn
+
         # 全局游标
-        if global_cursor is None:
-            global_cursor = conn.cursor()
-        
+        if global_conn is None:
+            global_conn = conn
+            cls.default_conn = conn
+
         return conn
-    
+
     @classmethod
     def switch(cls, name: str, db_name=None, inplace=False):
         ''' 切换数据库连接
@@ -157,7 +185,7 @@ class imysql:
         :return cursor
         '''
 
-        global global_cursor
+        global global_conn
         
         if name.find('.') > -1:
             name, db_name = name.split('.')
@@ -168,22 +196,28 @@ class imysql:
         
         # 连接
         conn = connections[name]
+
+        # 加线程锁
+        thread_lock.acquire(timeout=locked_timeout)
+
         # 断线重连
         conn.ping(reconnect=True)
-        # 游标
-        cursor = conn.cursor()
 
         # 切换到同连接的其他数据库
         if db_name is not None:
             conn.select_db(db_name)
-        
+
+        # 释放锁
+        thread_lock.release()
+
         # 全局游标
         if inplace is True:
-            global_cursor = cursor
+            global_conn = conn
+            cls.default_conn = conn
             return cls
         else:
             # 实例化
-            instance = cls(cursor=cursor)
+            instance = cls(conn=conn)
             # 动态修改 table 方法
             instance.table = instance._table
             # 动态修改 execute 方法
@@ -208,24 +242,28 @@ class imysql:
 
         :param sql
         :param args: sql 参数
+        :param fetch: False：返回 self，True：返回 list
         :return imysql 或 result
         '''
 
         instance = cls()
-        return instance._execute(sql=sql, args=args, fetch=fetch)
+        return instance._execute(sql, args=args, fetch=fetch)
 
     def _execute(self, sql: str, args=None, fetch=False):
         ''' 执行原生SQL
 
         :param sql
         :param args: sql 参数
+        :param fetch: False：返回 self，True：返回 list
         :return imysql 或 result
         '''
-        
-        sql = global_cursor.mogrify(sql, args)
+
+        thread_lock.acquire(timeout=locked_timeout)
+
+        sql = self.cursor.mogrify(sql, args)
 
         operation = sql.split(' ')[0].strip().lower()
-        
+
         if operation in ['insert', 'replace', 'update', 'delete', 'truncate', 'create', 'drop', 'alter']:
             with transaction.atomic(self.conn):
                 self.cursor.execute(sql)
@@ -233,21 +271,120 @@ class imysql:
                 # 记录SQL信息
                 cache_data['last_sql'] = sql
                 cache_data['last_operation'] = operation
-                cache_data['last_insert_id'] = global_cursor.connection.insert_id()
-                cache_data['effected_rows'] = global_cursor.rowcount
+                cache_data['last_insert_id'] = self.conn.insert_id()
+                cache_data['effected_rows'] = self.cursor.rowcount
 
+                thread_lock.release()
                 return cache_data['last_insert_id'] if operation == 'insert' else cache_data['effected_rows']
         else:
-            self.raw_sql = sql
-        
+            try:
+                self.cursor.execute(sql)
+            except Exception as e:
+                # 释放锁
+                thread_lock.release()
+                raise e
+
             # 记录SQL信息
+            self.raw_sql = sql
             cache_data['last_operation'] = 'select'
             cache_data['last_sql'] = sql
             
             if fetch is False:
-                return self
+                result = self
             else:
-                return self.all(fetch=True)
+                result = self.cursor.fetchall()
+
+            thread_lock.release()
+            return result
+
+    @classmethod
+    def execute_cross(cls, sql: str, chunk_size=500):
+        ''' 执行跨库（连接）查询
+
+        :param sql
+        :return generator
+        '''
+        
+        sql_list, bridging, column_alias = dqlparse.DqlParse().split_sql(sql)
+        
+        # 需要的字段
+        needs = {key: '' for key in column_alias}
+
+        flag = True
+        for name, sql in sql_list.items():
+            if flag is True:
+                flag = False
+                cursor = imysql.switch(name).execute(sql).all()
+
+                if len(sql_list) == 1:
+                    yield cursor
+                    break
+            else:
+                join_type, join_condition = bridging.get(name)
+
+                chunk = dict()
+                codes_mapping = dict()
+
+                for item in cursor:
+                    keys = []
+                    for t in join_condition:
+                        value = str(item.get(t[0]))
+                        keys.append(value)
+                        if t[0] not in codes_mapping:
+                            codes_mapping[t[0]] = []
+                        codes_mapping[t[0]].append(value)
+                    unique_key = '_'.join(keys)
+                    chunk[unique_key] = item
+                    
+                    if len(chunk) == chunk_size:
+                        yield cls.handle_cross_chunk(needs, chunk, codes_mapping, sql, join_type, join_condition, name)
+                        chunk.clear()
+                        codes_mapping.clear()
+
+                if len(chunk) > 0:
+                    yield cls.handle_cross_chunk(needs, chunk, codes_mapping, sql, join_type, join_condition, name)
+                    chunk.clear()
+                    codes_mapping.clear()
+
+    @classmethod
+    def handle_cross_chunk(cls, needs: dict, chunk: dict, codes_mapping: dict, sql: str, join_type: str, join_condition: list, name: str):
+        ''' 执行跨库（连接）查询 - 块处理 '''
+
+        for t in join_condition:
+            codes = "','".join(codes_mapping.get(t[0]))
+            sql += f" AND {t[1]} IN ('{codes}')"
+
+        sql = sql.replace('WHERE 1  AND', 'WHERE')
+
+        extra_data = imysql.switch(name).execute(sql).all()
+        
+        extra_mapping = dict()
+
+        for item in extra_data:
+            keys = []
+            for t in join_condition:
+                keys.append(str(item.get(t[0])))
+            unique_key = '_'.join(keys)
+            extra_mapping[unique_key] = item
+
+        join_type = join_type.upper()
+
+        for unique_key, item in chunk.items():
+            extra = extra_mapping.get(unique_key)
+            if extra is not None:
+                chunk[unique_key].update(extra)
+
+            # 过滤字段
+            out = needs.copy()
+            for k, v in chunk[unique_key].items():
+                if k in needs:
+                    out[k] = v
+
+            if join_type == 'LEFT':
+                yield out
+            elif join_type in ['RIGHT', 'INNER']:
+                if extra is not None:
+                    yield out
 
     @classmethod
     def gen_table(cls, table: str, alias=''):
@@ -660,9 +797,9 @@ class imysql:
         '''
 
         sql = self.get_raw_sql()
-        self.cursor.execute(sql)
+        self._execute(sql)
         self.reset_data()
-        
+
         if fetch is True:
             return self.cursor.fetchall()
         else:
@@ -677,7 +814,7 @@ class imysql:
         '''
 
         result = dict()
-        
+
         for item in self.all():
             if value is None:
                 result[item.get(key)] = item
@@ -691,11 +828,11 @@ class imysql:
 
         self.skip(num=0)
         self.limit(num=1)
-        
+
         sql = self.get_raw_sql()
-        self.cursor.execute(sql)
+        self._execute(sql)
         self.reset_data()
-        
+
         return self.cursor.fetchone()
 
     def scalar(self):
@@ -728,7 +865,7 @@ class imysql:
             self.data['fields'] = 'count(*) as ct'
         
         sql = self.get_raw_sql(wrapper)
-        self.cursor.execute(sql)
+        self._execute(sql)
         self.reset_data()
         one = self.cursor.fetchone()
         return one.get('ct') if one else False
@@ -741,13 +878,16 @@ class imysql:
         :param verify: 是否验证数据合法性，默认 True
         :return 影响行数，当 return_insert_id = True 时，返回 insert_id
         '''
-        
+
         if verify is True and self.__class__.check_validity(data) is False:
             raise exceptions.RuntimeError((403, '插入内容中包含非法字符'))
 
+        # 加线程锁
+        thread_lock.acquire(timeout=locked_timeout)
+
         # 开启事务处理
         with transaction.atomic(self.conn):
-            
+
             table = self.data.get('table')
             fields = self.gen_fields(data)
             values = self.gen_values(data)
@@ -755,14 +895,21 @@ class imysql:
             placeholder = "(%s)" % (','.join(types))
 
             sql = f'INSERT INTO {table} {fields} VALUES {placeholder}'
-            
-            effected_rows = self.cursor.executemany(sql, values)
+
+            try:
+                effected_rows = self.cursor.executemany(sql, values)
+            except Exception as e:
+                thread_lock.release()
+                raise e
+
             insert_id = self.conn.insert_id()
             # 记录SQL信息
             cache_data['last_operation'] = 'insert'
             cache_data['last_sql'] = sql
             cache_data['effected_rows'] = effected_rows
             cache_data['last_insert_id'] = insert_id
+            # 释放锁
+            thread_lock.release()
             # 返回插入的ID或影响的行数
             return insert_id if return_insert_id else effected_rows
 
@@ -798,7 +945,7 @@ class imysql:
                 limit=" LIMIT " + str(limit) if limit > 0 else ''
             )
 
-            effected_rows = self.cursor.execute(sql)
+            effected_rows = self._execute(sql)
 
             # 记录SQL信息
             cache_data['last_operation'] = 'update'
@@ -834,7 +981,7 @@ class imysql:
                 limit=" LIMIT " + str(limit) if limit > 0 else ''
             )
             
-            effected_rows = self.cursor.execute(sql)
+            effected_rows = self._execute(sql)
 
             # 记录SQL信息
             cache_data['last_operation'] = 'delete'
@@ -855,7 +1002,16 @@ class imysql:
     def get_effected_rows():
         return cache_data.get('effected_rows', 0)
 
-    @classmethod
-    @property
-    def default_conn(cls):
-        return global_cursor.connection
+    @staticmethod
+    def close(name=None):
+        if name is not None:
+            if name in connections:
+                conn = connections.get(name)
+                conn.cursor().close()
+                conn.close()
+                del connections[name]
+        else:
+            for name, conn in connections.items():
+                conn.cursor().close()
+                conn.close()
+            connections.clear()
